package edu.travel.order.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import edu.travel.order.entity.ShopOrderItem;
import edu.travel.order.service.ShopOrderItemService;
import edu.travel.remote.dto.AddOrderItemDto;
import edu.travel.remote.dto.GetOrderItemDto;
import edu.travel.remote.order.ShopOrderItemRemoteController;
import edu.travel.remote.vo.OrderItemVo;
import edu.travel.remote.vo.ShopSnapshotVo;
import edu.travel.remote.vo.SpecValueVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.web.BaseController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static edu.travel.rpc.RPCBaseResponse.success;


@RestController
@RequestMapping("/shopOrderItem")
public class ShopOrderItemController extends BaseController<ShopOrderItem> implements ShopOrderItemRemoteController {

    @Autowired
    private ShopOrderItemService shopOrderItemService;

    /**
     *  获取订单商品详情
     * @param params
     * @return {@link RPCBaseResponse }<{@link OrderItemVo }>
     */

    @Override
    @GetMapping("/getOrderItemVo")
    public RPCBaseResponse<OrderItemVo> getOrderItemVo(@RequestBody GetOrderItemDto params) {

        LambdaQueryChainWrapper<ShopOrderItem> query = shopOrderItemService.lambdaQuery().eq(ShopOrderItem::getSkuId, params.getSkuId())
                .eq(ShopOrderItem::getOrderId, params.getOrderId())
                .eq(ShopOrderItem::getProductId, params.getProductId());
        ShopOrderItem shopOrderItem = query.one();
        OrderItemVo bean = BeanUtil.toBean(shopOrderItem, OrderItemVo.class);
        if(!ObjectUtil.isEmpty(shopOrderItem) && !ObjectUtil.isEmpty(shopOrderItem.getSnapshot())){
            ShopSnapshotVo bean1 = JSONUtil.toBean(shopOrderItem.getSnapshot(), ShopSnapshotVo.class);
            bean.setShopSnapshotVo(bean1);
        }
        return success(bean);
    }

    /**
     *  添加订单商品信息
     * @param params
     * @return {@link RPCBaseResponse }<{@link Void }>
     */

    @Override
    @PostMapping("/addShopOrderItem")
    public RPCBaseResponse<Void> addShopOrderItem(@RequestBody List<AddOrderItemDto> params) {
        shopOrderItemService.addShopOrderItem(params);
        return success();
    }


}
